%{
/* PAS.Y: Pequeno analisador gramatical de programas PASCAL */

#include <stdio.h>

%}

%union {
	 double dval;
       int    ival;
       char   cval;
	 char   *sval;
}

%token <cval> ASSIGNMENT 
%token <cval> COLON 
%token <cval> COMMA 
%token <ival> DIGSEQ
%token <cval> DOT 
%token <cval> LPAREN 
%token <cval> MINUS 
%token <cval> PLUS 
%token <dval> REALNUMBER 
%token <cval> RPAREN 
%token <cval> SEMICOLON 
%token <sval> IDENTIFIER 
%token <sval> KW_BEGIN   
%token <sval> KW_END 
%token <sval> KW_PROGRAM 
%token <sval> KW_VAR 
%token <cval> ILLEGAL

%start program

%type <sval> identifier
%type <sval> type_denoter
%type <dval> value
%type <dval> unsigned_number 
%type <ival> unsigned_integer
%type <dval> unsigned_real
%type <sval> type_denoter
%type <sval> variable_access 
%type <dval> expression
%%

program : program_heading semicolon block DOT
    ;

program_heading : KW_PROGRAM identifier
    { printf ("PROGRAMA %s\n",$2); }
    ;

identifier_list : identifier_list comma identifier
    | identifier
    {  }
    ;

block : variable_declaration_part statement_part
    ;

variable_declaration_part : KW_VAR variable_declaration_list semicolon
    { } 
    ;

variable_declaration_part : KW_VAR variable_declaration_list
    { }
    ;

variable_declaration_list :
      variable_declaration_list semicolon variable_declaration
    | variable_declaration
    ;

/* vardecl: identifier or identifier_list */
variable_declaration : identifier COLON type_denoter
    { printf ("CRIAR: %s - Tipo: %s\n",$1,$3); }
    ;

type_denoter : identifier
    { $$ = $1; } 
    ;

statement_part : compound_statement ;

compound_statement : KW_BEGIN statement_sequence KW_END 
    { printf("FIM DO PROGRAMA\n"); }
    ;

statement_sequence : statement_sequence semicolon statement
    | statement
    | error
    { printf("\nstatement ignored\n"); /* yyclearin */ }
    ;

statement : assignment_statement
    | procedure_statement
    |
    ;

assignment_statement : variable_access ASSIGNMENT expression
    { printf("ATRIBUI para <%s> o valor %.2lf\n",$1,$3); }
    ;

variable_access : identifier
    { $$=$1; } 
    ;

procedure_statement : identifier params
    {  }
    ;

procedure_statement : identifier
    {  }
    ;

params : LPAREN actual_parameter_list RPAREN
    { }
    ;

actual_parameter_list : actual_parameter_list comma actual_parameter
    | actual_parameter
    ;

actual_parameter : value
    {  }
    ;

actual_parameter : identifier
    {  }
    ;

expression : value
    { $$=$1; }
    ;

value : sign unsigned_number 
    { $$=-1*$2; }
    ;

value : unsigned_number
    { $$=$1; }
    ;

unsigned_number : unsigned_integer
    { $$=$1; }
    ;

unsigned_number : unsigned_real
    { $$=$1; }
    ;

unsigned_integer : DIGSEQ
    { $$=$1; }
    ;

unsigned_real : REALNUMBER
    { $$=$1; } 
    ;

sign : PLUS
    {  }
    ;

sign : MINUS
    {  }
    ;

identifier : IDENTIFIER
    { $$ = $1; }
    ;

semicolon : SEMICOLON
    {  }
    ;

comma : COMMA
    {  }
    ;

%%

#include <stdio.h>

extern FILE *yyin;

yyerror(s)
char *s;
{
  fprintf(stderr,"ERROR: %d\n",s);
}

main ()
{
  do { yyparse(); }
  while (!feof(yyin));
}
